// NSMatrix+DragDrop.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// 
// 
// $Author: bjoern $
// $Locker:  $
// $Revision: 1.1.1.1 $
// $Date: 2003/10/16 09:37:13 $

#include <AppKit/AppKit.h>
#include "Controller.h"
#include "DragDropMatrix.h"

NSString * const DDMOperationConcludedNotification =
@"DDMOperationConcludedNotification";

static NSMutableArray *contactRows;

@implementation NSBrowserCell (DragAndDrop)
- (NSImage *) getImageOfContents: (NSSize) a_frame
{
  NSImage *image = nil;
  NSRect textFrame;
  
  textFrame.size = a_frame;
  textFrame.origin = NSZeroPoint;
  
  image = [[NSImage alloc] initWithSize: textFrame.size];
  [image setBackgroundColor:[NSColor clearColor]];
  
  [image lockFocus];
  [self drawInteriorWithFrame: textFrame inView: [NSView focusView]];
  [image unlockFocus];

  return [image autorelease];
}
@end

@implementation DragDropMatrix
- (BOOL) acceptsFirstMouse: (NSEvent *) theEvent
{
  return YES;
}

- (void) copyToPasteboard: (NSPasteboard *) a_pboard
{
  NSString *title = [[self selectedCell] title];
  NSAttributedString *attTitle = [[self selectedCell] attributedStringValue];
  
  [a_pboard declareTypes: [NSArray arrayWithObjects: NSRTFPboardType,
				   NSStringPboardType, nil]
	    owner: self]; 
  
  [a_pboard setData: [attTitle RTFFromRange: NSMakeRange(0, [attTitle length])
			       documentAttributes: nil]
	    forType: NSRTFPboardType];
  [a_pboard setString: title forType: NSStringPboardType];
}

- (void) mouseDown: (NSEvent *) event
{
  int row = -1, column = -1;

  [self registerForDraggedTypes: [NSArray arrayWithObjects: NSStringPboardType,
					  nil]];
  if([self getRow: &row
	   column: &column
	   forPoint: [self convertPoint:[event locationInWindow]
			   fromView: nil]] &&
     [[self cellAtRow:row column: column] isEnabled] &&
     [event modifierFlags] == 0)
    {
      NSArray *sel = [self selectedCells];
  
      if([sel count] == 1 ||
	 ![sel containsObject: [self cellAtRow: row column: column]])
	[self deselectAllCells];
      if([sel containsObject: [self cellAtRow: row column: column]])
	_didDrag = NO;
      [self selectCellAtRow: row column: column];
      [self sendAction];
    }
  else
    {
      if(![[self cellAtRow: row column: column] isLeaf])
	[self deselectAllCells];
      [super mouseDown: event];
    }
}

- (void) mouseUp: (NSEvent*) event
{
  int row, column;
  if(!_didDrag && [event modifierFlags] == 0 &&
     [[self selectedCell] isLeaf])
    {
      [self getRow: &row
	    column: &column
	    forPoint: [self convertPoint:[event locationInWindow]
			    fromView: nil]];
      [self deselectAllCells];
      [self selectCellAtRow: row column: column];
      [self sendAction];
    }

  [super mouseUp: event];
}

- (void) mouseDragged: (NSEvent*) event
{
  NSPasteboard *pboard;
  NSImage *image = nil;
  NSPoint dragPoint = NSZeroPoint;
  NSPoint cellOrigin;
  int width=0, height=0;

  NSArray *cells = [self selectedCells];
  int i;
  for(i=0; i<[cells count]; i++)
    if(![[cells objectAtIndex: i] isLeaf]) // can only drag leaf cells
      return;

  _didDrag = YES;
  
  pboard = [NSPasteboard pasteboardWithName: NSDragPboard];
  [self copyToPasteboard: pboard];
  
  dragPoint = [self convertPoint: [event locationInWindow]
		    fromView: nil];

  [contactRows release];
  contactRows = [[NSMutableArray alloc] initWithCapacity: [cells count]];
  for(i=0; i<[cells count]; i++)
    {
      int row, column;
      [self getRow: &row column: &column ofCell: [cells objectAtIndex: i]];
      [contactRows addObject: [NSNumber numberWithInt: row]];
    }

  cellOrigin = [self cellFrameAtRow: [self selectedRow]
		     column: [self selectedColumn]].origin;

  image = [self draggingImage];
  width = [image size].width;
  height = [image size].height;

  dragPoint.x -= width * (dragPoint.x - cellOrigin.x) / width;
  dragPoint.y += height * (cellOrigin.y + height - dragPoint.y) / height;

  [self dragImage: image
	at: dragPoint
	offset: NSMakeSize(0,0)
	event: event
	pasteboard: pboard
	source: self
	slideBack: YES];
}

- (NSDragOperation) draggingSourceOperationMaskForLocal: (BOOL) local
{
  return NSDragOperationLink;
}

- (void) draggingExited:  (id<NSDraggingInfo>) sender
{
  if(oldCell)
    {
      [self lockFocus];
      [oldCell drawWithFrame: oldFrame inView: self];
      [oldCell release];
      oldCell = nil;
      [_window flushWindow];
      [self unlockFocus];
    }
}

- (NSDragOperation) draggingUpdated: (id<NSDraggingInfo>) sender
{
  int row, column;
  NSRect frame;
  NSBrowserCell *cell;

  NSPoint p = [self convertPoint: [sender draggingLocation]
		    fromView: nil];

  if(![self getRow: &row column: &column forPoint: p] ||
     self == [sender draggingSource] ||
     row == 0)
    {
      if(oldCell)
	{
	  [self lockFocus];
	  [oldCell drawWithFrame: oldFrame inView: self];
	  [oldCell release];
	  oldCell = nil;
	  [_window flushWindow];
	  [self unlockFocus];
	}

      groupRow = -1;
      return NSDragOperationNone;
    }

  groupRow = row;
  cell = [self cellAtRow: row column: column];
  if(cell == [self selectedCell]) // can't drag to current group
    return NSDragOperationNone;
  
  frame = [self cellFrameAtRow: row column: column];

  [self lockFocus];
  if(cell != oldCell)
    {
      if(oldCell)
	{
	  NSRectFill(oldFrame);
	  [oldCell drawWithFrame: oldFrame inView: self];
	  [oldCell release];
	}
      oldCell = [cell retain];
      oldFrame = frame;
    }
  
  [[NSColor darkGrayColor] set];
  NSFrameRect(frame);
  [_window flushWindow];
  [self unlockFocus];

  return NSDragOperationLink;
}

- (BOOL) prepareForDragOperation: (id<NSDraggingInfo>) sender
{
  return YES;
}

- (BOOL) performDragOperation: (id<NSDraggingInfo>) sender
{
  if(oldCell)
    {
      [self lockFocus];
      [oldCell drawWithFrame: oldFrame inView: self];
      [oldCell release];
      oldCell = nil;
      [_window flushWindow];
      [self unlockFocus];
    }

  [[NSNotificationCenter defaultCenter]
    postNotificationName: DDMOperationConcludedNotification
    object: self
    userInfo: [NSDictionary dictionaryWithObjectsAndKeys:
			      contactRows,
			    @"SourceRows",
			    [NSNumber numberWithInt: groupRow],
			    @"DestinationRow",
			    nil]];
  return YES;
}

- (NSImage*) draggingImage
{
  NSArray *cells = [self selectedCells];
  int i;
  NSSize s, cellSize;
  s = cellSize = [self cellSize];
  int numCells = [cells count];
  if(numCells>3) numCells = 3;

  s.width += (numCells-1)*5;
  s.height += (numCells-1)*5;

  NSImage *img = [[[NSImage alloc] initWithSize: s] autorelease];
  [img lockFocus];

  int y = (numCells-1)*5; int x = (numCells-1)*5;
  for(i=0; i<numCells-1; i++,y-=5,x-=5)
    {
      [[NSColor colorWithDeviceRed: 1.0/(numCells-i)
		green: 1.0/(numCells-i)
		blue: 1.0/(numCells-i)
		alpha: 1.0] set];
      NSRect r = NSMakeRect(x, y, cellSize.width, cellSize.height);
      NSRectFill(r);
    }

  NSImage *img2 = [[cells objectAtIndex: 0]
		    getImageOfContents: [self cellSize]];
  [img2 dissolveToPoint: NSZeroPoint fraction: 1.0];

  return img;
}

@end
