// ADPerson.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Address Book Framework for GNUstep
// 
// $Author: bjoern $
// $Locker:  $
// $Revision: 1.1.1.1 $
// $Date: 2003/10/16 09:37:07 $

/* system includes */
/* (none) */

/* my includes */
#include "ADAddressBook.h"
#include "ADPerson.h"
#include "ADGlobals.h"
#include "ADTypedefs.h"
#include "ADMultiValue.h"

static NSMutableDictionary *_propTypes;
static ADScreenNameFormat _scrNameFormat = ADScreenNameLastNameFirst;
#define N(x) [NSNumber numberWithInt: x]

@implementation ADPerson
+ (void) initialize
{
  _propTypes = [[NSMutableDictionary alloc]
		 initWithObjectsAndKeys:
		   N(ADStringProperty),        ADUIDProperty,
		 N(ADDateProperty),            ADCreationDateProperty,
		 N(ADDateProperty),            ADModificationDateProperty,
		 N(ADStringProperty),          ADSharedProperty,
		 N(ADStringProperty),          ADFirstNameProperty,
		 N(ADStringProperty),          ADLastNameProperty,
		 N(ADStringProperty),          ADFirstNamePhoneticProperty,
		 N(ADStringProperty),          ADLastNamePhoneticProperty,
		 N(ADDateProperty),            ADBirthdayProperty,
		 N(ADStringProperty),          ADOrganizationProperty,
		 N(ADStringProperty),          ADJobTitleProperty,
		 N(ADStringProperty),          ADHomePageProperty,
		 N(ADMultiStringProperty),     ADEmailProperty,
		 N(ADMultiDictionaryProperty), ADAddressProperty,
		 N(ADMultiStringProperty),     ADPhoneProperty,
		 
		 N(ADMultiStringProperty),     ADAIMInstantProperty,
		 N(ADMultiStringProperty),     ADJabberInstantProperty,
		 N(ADMultiStringProperty),     ADMSNInstantProperty,
		 N(ADMultiStringProperty),     ADYahooInstantProperty,
		 N(ADMultiStringProperty),     ADICQInstantProperty,
		 
		 N(ADStringProperty),          ADNoteProperty,
		 
		 N(ADStringProperty),          ADMiddleNameProperty,
		 N(ADStringProperty),          ADMiddleNamePhoneticProperty,
		 N(ADStringProperty),          ADTitleProperty,
		 N(ADStringProperty),          ADSuffixProperty,
		 N(ADStringProperty),          ADNicknameProperty,
		 N(ADStringProperty),          ADMaidenNameProperty,

		 N(ADDataProperty),            ADImageProperty,
		 N(ADStringProperty),          ADImageTypeProperty,
		 nil];

  NSDictionary *dom = [[NSUserDefaults standardUserDefaults]
			persistentDomainForName: @"Addresses"];
  NSString *scrName;
  if(!dom)
    {
      _scrNameFormat = ADScreenNameLastNameFirst;
      scrName = @"LastNameFirst";
    }
  else
    {
      scrName = [dom objectForKey: @"ScreenNameFormat"];
      if(!scrName || [scrName isEqualToString: @"LastNameFirst"])
	{
	  _scrNameFormat = ADScreenNameLastNameFirst;
	  scrName = @"LastNameFirst";
	}
      else if([scrName isEqualToString: @"FirstNameFirst"])
	{
	  _scrNameFormat = ADScreenNameFirstNameFirst;
	  scrName = @"FirstNameFirst";
	}
      else
	{
	  NSLog(@"Unknown value %@ for ScreenNameFormat. "
		@"Using LastNameFirst.\n");
	  _scrNameFormat = ADScreenNameFirstNameFirst;
	  scrName = @"LastNameFirst";
	}
    }

  dom = [NSDictionary dictionaryWithObjectsAndKeys: scrName,
		      @"ScreenNameFormat", nil];
  [[NSUserDefaults standardUserDefaults] setPersistentDomain: dom
					 forName: @"Addresses"];
}

+ (int) addPropertiesAndTypes: (NSDictionary*) properties
{
  int retval = 0;
  NSEnumerator *e = [properties keyEnumerator];
  NSString *key;
  while((key = [e nextObject]))
    if(![_propTypes objectForKey: key])
      {
	[_propTypes setObject: [properties objectForKey: key]
		    forKey: key];
	retval++;
      }
  return retval;
}

+ (int) removeProperties: (NSArray*) properties
{
  int retval = 0;
  NSEnumerator *e = [properties objectEnumerator];
  NSString* key;
  while((key = [e nextObject]))
    if([_propTypes objectForKey: key])
      {
	[_propTypes removeObjectForKey: key];
	retval++;
      }
  return retval;
}
  
+ (NSArray*) properties
{
  return [_propTypes allKeys];
}

+ (ADPropertyType) typeOfProperty: (NSString*) property
{
  id val = [_propTypes objectForKey: property];
  if(val) return (ADPropertyType)[val intValue];
  return ADErrorInProperty;
}

+ (ADSearchElement*) searchElementForProperty: (NSString*) property 
				       label: (NSString*) label 
					 key: (NSString*) key 
				       value: (id) value 
				  comparison: (ADSearchComparison) comparison
{
  return [[[ADRecordSearchElement alloc]
	    initWithProperty: property
	    label: label
	    key: key
	    value: value
	    comparison: comparison]
	   autorelease];
}

- (id) valueForProperty: (NSString*) property
{
  id val = [super valueForProperty: property];
  ADPropertyType type = [[self class] typeOfProperty: property];

  // multi-value? If so, create empty one and put it in
  if(!val && (type & ADMultiValueMask))
    {
      val = [[[ADMutableMultiValue alloc] initWithType: type] autorelease];
      [_dict setObject: val forKey: property];
    }

  return val;
}

- (NSArray*) parentGroups
{
  if(![self addressBook])
    return [NSArray array];
  return [[self addressBook] groupsContainingRecord: self];
}

- (id) initWithVCardRepresentation: (NSData*) vCardData
{
  return [self initWithRepresentation: vCardData type: @"vcf"];
}

- (NSData *) vCardRepresentation
{
  return [self representationWithType: @"vcf"];
}
@end

@implementation ADPerson (AddressesExtensions)
+ (ADScreenNameFormat) screenNameFormat
{
  return _scrNameFormat;
}

+ (void) setScreenNameFormat: (ADScreenNameFormat) aFormat
{
  _scrNameFormat = aFormat;
}

- (NSString*) screenName
{
  NSString *last, *first;

  last = [self valueForProperty: ADLastNameProperty];
  first = [self valueForProperty: ADFirstNameProperty];
  if(!last && !first)
    return @"New Person";
  else if(!first) return last;
  else if(!last) return first;
  else if(_scrNameFormat == ADScreenNameFirstNameFirst)
    return [NSString stringWithFormat: @"%@ %@", first, last];
  else
    return [NSString stringWithFormat: @"%@, %@", last, first];
}

- (int) compareByScreenName: (ADPerson*) theOtherGuy
{
  if([[self screenName] isEqualToString: @"New Person"])
    return -1;
  else if([[theOtherGuy screenName] isEqualToString: @"New Person"])
    return 1;
  return [[self screenName] compare: [theOtherGuy screenName]];
}
@end

